/* 
 * File:   json_parser.c
 * Author: Arthur Brandao
 *
 * Created on 29 octobre 2018
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "json.h"

/* --- Fonctions privée --- */

/**
 * Ajout un noeud au JsonEncoder
 * @param JsonEncoder* La structure pour encoder 
 * @param char* La chaine à mettre dans le noeud
 */
void add_json_node(JsonEncoder* this, char* str){
    //Création node
    JsonNode* node;
    node = malloc(sizeof(JsonNode));
    //Allocation node
    int length = strlen(str) + 1;
    node->str = malloc(length * sizeof(char));
    memset(node->str, 0, length);
    strncpy(node->str, str, length - 1);
    //Si 1er node
    if(this->head == NULL){
        this->head = node;
        node->prev = NULL;
    } else {
        node->prev = this->tail;
        node->prev->next = node;
    }
    this->tail = node;
    node->next = NULL;
}

/**
 * Supprimme un noeud
 * @param JsonNode* Le noeud à supprimer
 */
void delete_json_node(JsonNode* node){
    free(node->str);
}

/* --- Fonctions publique --- */

void ini_encoder(JsonEncoder* this){
    this->head = NULL;
    this->tail = NULL;
    this->length = 0;
}

void add_value(JsonEncoder* this, char* str){
    //Ajoute la longueur de la chaine au total
    this->length += strlen(str) + 2; //Chaine + ", "
    //Ajoute le noeud
    add_json_node(this, str);
}

void add_string(JsonEncoder* this, char* key, char* val){
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(val) + 4 + 2 + 1; //clef + val + 4 guillemet + ": " + \0
    char* str = malloc(length * sizeof(char)); 
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    str[index++] = '"';
    for(int i = 0; i < strlen(val); i++, index++){
        str[index] = val[i];
    }
    str[index++] = '"';
    //Ajout
    add_value(this, str);
    free(str);
}

void add_number(JsonEncoder* this, char* key, double val, int ndigit){
    //Double en string
    char nombre[20];
    ftoa(val, nombre, ndigit);
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(nombre) + 2 + 2 + 1; //clef + val + 2 guillemets + ": " + \0
    char* str = malloc(length * sizeof(char));
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    for(int i = 0; i < strlen(nombre); i++, index++){
        str[index] = nombre[i];
    }
    //Ajout
    add_value(this, str);
    free(str);
}

void add_integer(JsonEncoder* this, char* key, int val){
    //Double en string
    char nombre[20];
    snprintf(nombre, 20, "%d", val);
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(nombre) + 2 + 2 + 1; //clef + val + 2 guillemets + ": " + \0
    char* str = malloc(length * sizeof(char));
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    for(int i = 0; i < strlen(nombre); i++, index++){
        str[index] = nombre[i];
    }
    //Ajout
    add_value(this, str);
    free(str);
}

void add_boolean(JsonEncoder* this, char* key, boolean val){
    //On determine le boolean
    char bool[6];
    if(val){
        strcpy(bool, "true");
    } else {
        strcpy(bool, "false");
    }
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(bool) + 2 + 2 + 1; //clef + val + 2 guillemets + ": " + \0
    char* str = malloc(length * sizeof(char));
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    for(int i = 0; i < strlen(bool); i++, index++){
        str[index] = bool[i];
    }
    //Ajout
    add_value(this, str);
    free(str);
}

void add_array(JsonEncoder* this, char* key, JsonArray* val){
    //Verification
    if(val->mode != JSON_ARRAY_ENCODER){
        return;
    }
    //Recup string du JsonEncoder
    char* json;
    json = json_encode_array(val);
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(json) + 2 + 2 + 1; //clef + val + 2 guillemets + ": " + \0
    char* str = malloc(length * sizeof(char));
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    for(int i = 0; i < strlen(json); i++, index++){
        str[index] = json[i];
    }
    //Ajout
    add_value(this, str);
    free(str);
    free(json);
}

void add_object(JsonEncoder* this, char* key, JsonEncoder* val){
    //Recup string du JsonEncoder
    char* json;
    json = json_encode(val);
    //Creation chaine
    int index = 0, length = strlen(key) + strlen(json) + 2 + 2 + 1; //clef + val + 2 guillemets + ": " + \0
    char* str = malloc(length * sizeof(char));
    memset(str, 0, length);
    //Copie
    str[index++] = '"';
    for(int i = 0; i < strlen(key); i++, index++){
        str[index] = key[i];
    }
    str[index++] = '"';
    str[index++] = ':';
    str[index++] = ' ';
    for(int i = 0; i < strlen(json); i++, index++){
        str[index] = json[i];
    }
    //Ajout
    add_value(this, str);
    free(str);
    free(json);
}

char* json_encode(JsonEncoder* this){
    boolean first = true;
    int length, index = 0;
    //Allocation chaine
    char* str;
    str = malloc((this->length + 2) * sizeof(char)); // La chaine + {}
    memset(str, 0, this->length + 2);
    //Creation de la chaine
    JsonNode* node;
    node = this->head;
    str[index++] = '{';
    while(node != NULL){
        if(!first){
            str[index++] = ',';
            str[index++] = ' ';
        }
        length = strlen(node->str);
        for(int i = 0; i < length; i++, index++){
            str[index] = node->str[i];
        }
        if(first){
            first = false;
        }
        /*if(first){
            snprintf(str, this->length + 2, "%s%s", str, node->str);
            printf("Json %s\n", str);
            first = false;
        } else {
            snprintf(str, this->length + 2, "%s, %s", str, node->str);
            printf("Json %s\n", str);
        }*/
        node = node->next;
    }
    str[index++] = '}';
    //Retour
    return str;
}

void clean_json_encoder(JsonEncoder* this){
    //Parcours les noeuds et les supprimes
    JsonNode* node, * tmp;
    node = this->head;
    while(node != NULL){
        tmp = node->next;
        delete_json_node(node);
        free(node);
        node = tmp;
    }
    //Reset la structure
    this->head = NULL;
    this->tail = NULL;
    this->length = 0;
}